from ScoutSuite.providers.aliyun.resources.base import AliyunResources


class Buckets(AliyunResources):
    async def fetch_all(self):
        for raw_bucket in await self.facade.oss.get_buckets():
            id, bucket = self._parse_bucket(raw_bucket)
            self[id] = bucket

    def _parse_bucket(self, raw_bucket):
        bucket_dict = {}
        bucket_dict['name'] = bucket_dict['id'] = raw_bucket.name
        bucket_dict['location'] = raw_bucket.location
        bucket_dict['storage_class'] = raw_bucket.storage_class
        bucket_dict['creation_date'] = raw_bucket.creation_date
        return bucket_dict['id'], bucket_dict



from ScoutSuite.providers.azure.resources.base import AzureCompositeResources

from .auto_provisioning_settings import AutoProvisioningSettings
from .pricings import Pricings
from .security_contacts import SecurityContacts
from .information_protection_policies import InformationProtectionPolicies
from .settings import Settings


class SecurityCenter(AzureCompositeResources):
    _children = [
        (AutoProvisioningSettings, 'auto_provisioning_settings'),
        (Pricings, 'pricings'),
        (SecurityContacts, 'security_contacts'),
        (InformationProtectionPolicies, 'information_protection_policies'),
        (Settings, 'settings')
    ]

    async def fetch_all(self):
        await self._fetch_children(resource_parent=self)



from azure.graphrbac import GraphRbacManagementClient
from azure.mgmt.authorization import AuthorizationManagementClient

from ScoutSuite.core.console import print_exception
from ScoutSuite.providers.utils import run_concurrently


class GraphRBACFacade:
    def __init__(self, graphrbac_credentials, credentials, tenant_id, subscription_id):
        self._subscription_id = subscription_id
        self._client = GraphRbacManagementClient(graphrbac_credentials, tenant_id=tenant_id)
        self._authorization_client = AuthorizationManagementClient(credentials, subscription_id=subscription_id)

    async def get_users(self):
        try:
            return await run_concurrently(lambda: list(self._client.users.list()))
        except Exception as e:
            print_exception('Failed to retrieve users: {}'.format(e))
            return []

    async def get_groups(self):
        try:
            return await run_concurrently(lambda: list(self._client.groups.list()))
        except Exception as e:
            print_exception('Failed to retrieve groups: {}'.format(e))
            return []

    async def get_user_groups(self, user_id):
        try:
            return await run_concurrently(lambda: list(
                self._client.users.get_member_groups(object_id=user_id,
                                                     security_enabled_only=False))
                                          )
        except Exception as e:
            print_exception('Failed to retrieve user\'s groups: {}'.format(e))
            return []

    async def get_service_principals(self):
        try:
            return await run_concurrently(lambda: list(self._client.service_principals.list()))
        except Exception as e:
            print_exception('Failed to retrieve service principals: {}'.format(e))
            return []

    async def get_applications(self):
        try:
            return await run_concurrently(lambda: list(self._client.applications.list()))
        except Exception as e:
            print_exception('Failed to retrieve applications: {}'.format(e))
            return []

    async def get_roles(self):
        try:
            scope = '/subscriptions/{}'.format(self._subscription_id)
            return await run_concurrently(lambda: list(self._authorization_client.role_definitions.list(scope=scope)))
        except Exception as e:
            print_exception('Failed to retrieve roles: {}'.format(e))
            return []

    async def get_role_assignments(self):
        try:
            return await run_concurrently(lambda: list(self._authorization_client.role_assignments.list()))
        except Exception as e:
            print_exception('Failed to retrieve role assignments: {}'.format(e))
            return []



from ScoutSuite.providers.azure.resources.base import AzureResources


class Settings(AzureResources):
    async def fetch_all(self):
        for raw_settings in await self.facade.securitycenter.get_information_protection_policies():
            id, settings = self._parse_settings(
                raw_settings)
            self[id] = settings

    def _parse_settings(self, settings):
        settings_dict = {}
        #auto_provisioning_setting_dict['id'] = auto_provisioning_settings.id

        return settings_dict['id'], settings_dict



from ScoutSuite.providers.aliyun.authentication_strategy import AliyunCredentials

from ScoutSuite.providers.aliyun.utils import get_oss_client


class OSSFacade:
    def __init__(self, credentials: AliyunCredentials):
        self._credentials = credentials

    async def get_buckets(self):
        """
        Get all instances

        :return: a list of all instances
        """
        client = get_oss_client(credentials=self._credentials)
        response = client.list_buckets()  # TODO this doesn't follow standards
        if response:
            return response.buckets
        else:
            return []



from ScoutSuite.providers.azure.resources.base import AzureResources


class ServicePrincipals(AzureResources):
    async def fetch_all(self):
        for raw_service_principal in await self.facade.graphrbac.get_service_principals():
            id, service_principal = await self._parse_service_principal(raw_service_principal)
            # exclude built-in service principals
            if service_principal['publisher_name'] != 'Microsoft Services':
                self[id] = service_principal

    async def _parse_service_principal(self, raw_service_principal):
        service_principal_dict = {}
        service_principal_dict['id'] = raw_service_principal.object_id
        service_principal_dict['name'] = raw_service_principal.display_name
        service_principal_dict['additional_properties'] = raw_service_principal.additional_properties
        service_principal_dict['deletion_timestamp'] = raw_service_principal.deletion_timestamp
        service_principal_dict['object_type'] = raw_service_principal.object_type
        service_principal_dict['account_enabled'] = raw_service_principal.account_enabled
        service_principal_dict['alternative_names'] = raw_service_principal.alternative_names
        service_principal_dict['app_name'] = raw_service_principal.app_display_name
        service_principal_dict['app_id'] = raw_service_principal.app_id
        service_principal_dict['app_owner_tenant_id'] = raw_service_principal.app_owner_tenant_id
        service_principal_dict['app_role_assignment_required'] = raw_service_principal.app_role_assignment_required
        service_principal_dict['app_roles'] = raw_service_principal.app_roles
        service_principal_dict['error_url'] = raw_service_principal.error_url
        service_principal_dict['homepage'] = raw_service_principal.homepage
        service_principal_dict['key_credentials'] = raw_service_principal.key_credentials
        service_principal_dict['logout_url'] = raw_service_principal.logout_url
        service_principal_dict['oauth2_permissions'] = raw_service_principal.oauth2_permissions
        service_principal_dict['password_credentials'] = raw_service_principal.password_credentials
        service_principal_dict[
            'preferred_token_signing_key_thumbprint'] = raw_service_principal.preferred_token_signing_key_thumbprint
        service_principal_dict['publisher_name'] = raw_service_principal.publisher_name
        service_principal_dict['reply_urls'] = raw_service_principal.reply_urls
        service_principal_dict['saml_metadata_url'] = raw_service_principal.saml_metadata_url
        service_principal_dict['service_principal_names'] = raw_service_principal.service_principal_names
        service_principal_dict['service_principal_type'] = raw_service_principal.service_principal_type
        service_principal_dict['tags'] = raw_service_principal.tags

        service_principal_dict['roles'] = []  # this will be filled in `finalize()`

        return service_principal_dict['id'], service_principal_dict




from ScoutSuite.providers.aws.resources.base import AWSResources


class Roles(AWSResources):
    async def fetch_all(self):
        raw_roles = await self.facade.iam.get_roles()
        for raw_role in raw_roles:
            name, resource = self._parse_role(raw_role)
            self[name] = resource

    def _parse_role(self, raw_role):
        role_dict = {}
        role_dict['id'] = raw_role.get('RoleId')
        role_dict['name'] = raw_role.get('RoleName')
        role_dict['arn'] = raw_role.get('Arn')
        role_dict['description'] = raw_role.get('Description')
        role_dict['path'] = raw_role.get('Path')
        role_dict['create_date'] = raw_role.get('CreateDate')
        role_dict['max_session_duration'] = raw_role.get('MaxSessionDuration')
        role_dict['instance_profiles'] = raw_role.get('instance_profiles')
        role_dict['instances_count'] = raw_role.get('instances_count')
        role_dict['inline_policies'] = raw_role.get('inline_policies')
        role_dict['inline_policies_count'] = raw_role.get('inline_policies_count')
        role_dict['assume_role_policy'] = raw_role.get('assume_role_policy')
        return role_dict['id'], role_dict



from ScoutSuite.providers.aliyun.resources.base import AliyunCompositeResources
from ScoutSuite.providers.aliyun.resources.oss.buckets import Buckets


class OSS(AliyunCompositeResources):
    _children = [
        (Buckets, 'buckets')
    ]

    async def fetch_all(self, **kwargs):
        await self._fetch_children(resource_parent=self)
